@@ -5,6 +5,7 @@ try:
 except ImportError:
     import json
 
+from . import decorators, refine
 from .action import Action
 from .async_action import AsyncAction
 from .async_client import AsyncBotoy
@@ -24,3 +25,7 @@ __all__ = [
     'GroupMsg',
     'FriendMsg',
 ]
+
+from .__version__ import check_version as __check_version
+
+__check_version()
