@@ -1,3 +1,4 @@
+import collections
 import time
 import traceback
 from typing import List, Union
@@ -111,13 +112,26 @@ class Action:
         picUrl: str = '',
         picBase64Buf: str = '',
         fileMd5: str = '',
+        picMd5s: Union[str, List[str]] = '',
         flashPic=False,
         atUser: Union[int, List[int]] = 0,
     ) -> dict:
         """发送群组图片消息"""
-        assert any([picUrl, picBase64Buf, fileMd5]), '缺少参数'
+        assert any([picUrl, picBase64Buf, fileMd5, picMd5s]), '缺少参数'
         if atUser != 0:
             content = macro.atUser(atUser) + '\n' + content
+        if picMd5s:
+            if not isinstance(picMd5s, collections.Sequence):
+                picMd5s = [picMd5s]
+            return self.post(
+                'SendMsgV2',
+                {
+                    "ToUserUid": group,
+                    "SendToType": 2,
+                    "SendMsgType": "PicMsg",
+                    "PicMd5s": picMd5s,
+                },
+            )
         return self.post(
             'SendMsg',
             {
@@ -314,26 +328,101 @@ class Action:
             },
         )
 
+    def repostVideo2Group(self, group: int, forwordBuf: str) -> dict:
+        """转发视频到群聊
+        :param group: 群号
+        :param forwordBuf: 原视频的forwordBuf字段
+        """
+        return self.post(
+            'SendMsg',
+            {
+                "toUser": group,
+                "sendToType": 2,
+                "sendMsgType": "ForwordMsg",
+                "forwordBuf": forwordBuf,
+                "forwordField": 19,
+            },
+        )
+
+    def repostVideo2Friend(self, user: int, forwordBuf: str) -> dict:
+        """转发视频到好友
+        :param user: 好友QQ
+        :param forwordBuf: 原视频的forwordBuf字段
+        """
+        return self.post(
+            'SendMsg',
+            {
+                "toUser": user,
+                "sendToType": 1,
+                "sendMsgType": "ForwordMsg",
+                "forwordBuf": forwordBuf,
+                "forwordField": 19,
+            },
+        )
+
     ############获取############
-    def getCookies(self):
+    def getCookies(self) -> dict:
         """获取QQ相关cookie"""
         return self.get('GetUserCook')
 
-    def getUserList(self):  # todo:循环获取
-        """获取好友列表"""
-        return self.post('GetQQUserList', {'StartIndex': 0})
-
-    def getUserInfo(self, UserID: int):
+    def getUserInfo(self, user: int) -> dict:
         """获取任意用户信息昵称头像等"""
-        return self.post('GetUserInfo', {'UserID': UserID})
+        return self.post('GetUserInfo', {'UserID': user})
 
-    def getGroupList(self):  # todo:循环获取
-        """获取当前bot加入的群列表"""
-        return self.post('GetGroupList', {'NextToken': ''})
+    def getUserList(self) -> List[dict]:  # FIXME: 目前只适用于好友较少一次获取就可以的情况，好友较多需要循环获取
+        """获取好友列表"""
+        data = self.post('GetQQUserList', {'StartIndex': 0})
+        if 'Friendlist' in data:
+            return data['Friendlist']
+        return []
 
-    def getGroupUserList(self, group: int):  # todo:循环获取
-        """获取群信息(bot已经加入的,未加入的获取不到)"""
-        return self.post('GetGroupUserList', {'GroupUin': group, 'LastUin': ''})
+    def getGroupList(self) -> List[dict]:  # FIXME: 循环获取
+        """获取群列表"""
+        data = self.post('GetGroupList', {'NextToken': ''})
+        if 'TroopList' in data:
+            return data['TroopList']
+        return []
+
+    def getGroupMembers(self, group: int) -> List[dict]:
+        """获取群成员列表"""
+        members = []
+        lastUin = 0
+        while True:
+            data = self.post(
+                'GetGroupUserList', {"GroupUin": group, "LastUin": lastUin}
+            )
+            if 'MemberList' in data:
+                members.extend(data['MemberList'])
+            if 'LastUin' not in data or data['LastUin'] == 0:
+                break
+            lastUin = data['LastUin']
+            time.sleep(0.6)
+        return members
+
+    def getGroupAdminList(self, group: int, include_owner=True) -> List[dict]:
+        """获取群管理员列表
+        :param group: 群号
+        :param include_owner: 是否包括群主
+        """
+        members = self.getGroupMembers(group)
+        if include_owner:
+            # 获取群主id
+            for groupInfo in self.getGroupList():
+                if groupInfo['GroupId'] == group:
+                    owner = groupInfo['GroupOwner']
+                    break
+            admins = [
+                member
+                for member in members
+                if member['GroupAdmin'] == 1 or member['MemberUin'] == owner
+            ]
+        else:
+            admins = [member for member in members if member['GroupAdmin'] == 1]
+        return admins
+
+    def getClusterInfo(self) -> dict:
+        """获取当前集群信息"""
+        return self.get('', path='/v1/ClusterInfo')
 
     ############操作############
     def setUniqueTitle(self, user: int, group: int, title: str) -> dict:
@@ -429,6 +518,55 @@ class Action:
             {"GroupID": group, "MsgSeq": msgSeq, "MsgRandom": msgRandom},
         )
 
+    def inviteUserJoinGroup(self, group: int, user: int) -> dict:
+        """拉人入群
+        :param group: 哪个群?
+        :param user: 拉谁?
+        """
+        return self.post(
+            'GroupMgr',
+            {"ActionType": 8, "GroupID": group, "ActionUserID": user, "Content": ""},
+        )
+
+    def joinGroup(self, group: int, content: str = "") -> dict:
+        """加入群聊
+        :param group: 哪个群?
+        :param content: 加群理由
+        """
+        return self.post(
+            'GroupMgr',
+            {"ActionType": 1, "GroupID": group, "ActionUserID": 0, "Content": content},
+        )
+
+    def exitGroup(self, group: int) -> dict:
+        """退出群聊
+        :param group: 哪个群?
+        """
+        return self.post(
+            'GroupMgr',
+            {"ActionType": 2, "GroupID": group, "ActionUserID": 0, "Content": ""},
+        )
+
+    def driveUserAway(self, group: int, user: int) -> dict:
+        """移出群聊
+        :param group: 哪个群?
+        :param user:把谁踢出去?
+        """
+        return self.post(
+            'GroupMgr',
+            {"ActionType": 3, "GroupID": group, "ActionUserID": user, "Content": ""},
+        )
+
+    def refreshKeys(self) -> dict:
+        """刷新key二次登陆"""
+        return self.get('', path='/v1/RefreshKeys')
+
+    def logout(self, flag=False) -> bool:
+        """退出指定QQ
+        :param flag: 是否删除设备信息文件
+        """
+        return self.post('LogOut', {"Flag": flag})
+
     ############################################################################
     def baseRequest(
         self,
@@ -450,9 +588,13 @@ class Action:
                 method, httpx.URL(url=path, params=params), json=payload
             )
             resp.raise_for_status()
-        except httpx.RequestError as e:
-            if isinstance(e, httpx.Timeout):
-                logger.warning('响应超时，但不代表处理未成功, 结果未知!')
+        except httpx.HTTPError as e:
+            if isinstance(e, httpx.TimeoutException):
+                logger.warning(f'响应超时，但不代表处理未成功, 结果未知! => {e}')
+            elif isinstance(e, httpx.HTTPStatusError):
+                logger.error(
+                    f"响应码出错 => {resp.status_code}",
+                )
             else:
                 logger.error(f'请求出错: {traceback.format_exc()}')
             return {}
