@@ -409,6 +409,35 @@ class AsyncAction:
             {"GroupID": group, "MsgSeq": msgSeq, "MsgRandom": msgRandom},
         )
 
+    async def joinGroup(self, group: int, content: str = "") -> dict:
+        """加入群聊
+        :param group: 哪个群?
+        :param content: 加群理由
+        """
+        return await self.post(
+            'GroupMgr',
+            {"ActionType": 1, "GroupID": group, "ActionUserID": 0, "Content": content},
+        )
+
+    async def exitGroup(self, group: int) -> dict:
+        """退出群聊
+        :param group: 哪个群?
+        """
+        return await self.post(
+            'GroupMgr',
+            {"ActionType": 2, "GroupID": group, "ActionUserID": 0, "Content": ""},
+        )
+
+    async def driveUserAway(self, group: int, user: int) -> dict:
+        """移出群聊
+        :param group: 哪个群?
+        :param user:把谁踢出去?
+        """
+        return await self.post(
+            'GroupMgr',
+            {"ActionType": 3, "GroupID": group, "ActionUserID": user, "Content": ""},
+        )
+
     async def baseRequest(
         self,
         method: str,
@@ -429,9 +458,13 @@ class AsyncAction:
                 method, httpx.URL(url=path, params=params), json=payload
             )
             resp.raise_for_status()
-        except httpx.RequestError as e:
-            if isinstance(e, httpx.Timeout):
-                logger.warning('响应超时，但不代表处理未成功, 结果未知!')
+        except httpx.HTTPError as e:
+            if isinstance(e, httpx.TimeoutException):
+                logger.warning(f'响应超时，但不代表处理未成功, 结果未知! => {e}')
+            elif isinstance(e, httpx.HTTPStatusError):
+                logger.error(
+                    f"响应码出错 => {resp.status_code}",
+                )
             else:
                 logger.error(f'请求出错: {traceback.format_exc()}')
             return {}
